import numpy as np
from spacy import load as LOAD
from spacy.matcher import Matcher
from collections import Counter
import matplotlib.pyplot as plt
from spacy import util as util
import subprocess
from datetime import datetime


ngram_vars={
    "nlp":"",
    "matcher":""
}
plt.rcParams['figure.figsize'] = [8, 6]
plt.set_xticks(rotation=90)
def setup():
    global ngram_vars
    ngram_vars["nlp"] = LOAD('es_core_news_md', disable=['parser','ner','textcat','...'] )
    ngram_vars["matcher"] = Matcher(ngram_vars["nlp"].vocab)

    pattern_1 = [{"POS": "NOUN"},{"LOWER": "de"}, {"POS": "NOUN"}]
    pattern_2 = [{"POS": "NOUN"}, {"POS": "ADJ"}]

    ngram_vars["matcher"].add("NOUN-de-NOUN", [pattern_1])
    ngram_vars["matcher"].add("NOUN-ADJ", [pattern_2])   
    
if not util.is_package("es_core_news_md"):
    subprocess.run("python -m spacy download es_core_news_md")
    print("...instalando el modelo de leguanje 'es_core_news_md', porfavor reiniciar el kernel y vuelva a importar la libreria")
else:
    setup()


def get_keywords(dataFrame):
    """Add a new column to the dataFrame with all the keywords defined as NOUN or fit with the spacy Matcher.
    
     Args:
        dataFrame (dataFrame, required): dataFrame with news in PySophia2 structure
        
    Returns:
        None
    """
    new_row=np.zeros(len(dataFrame.index))
    dataFrame['keywords']=new_row
    for index,row in dataFrame.iterrows():
        text=row["text"]
        filteredText=''.strip()
        tokens = ngram_vars["nlp"](text.lower())
        '''tokens = [token
                 for token in tokens
                 if not token.is_stop and not token.is_punct]'''
        for token in tokens:
            if token.pos_ == "NOUN" :
                filteredText=filteredText+str(token.text.lower().strip())+","
                
        matches = ngram_vars["matcher"](tokens)
        for mach_id, start, end in matches:
            span = tokens[start:end]
            filteredText=filteredText+str(span.text.lower().strip())+","
        dataFrame.loc[index,'keywords']=filteredText
        
def get_frequency(df):
    
    """Create a dictionary with the frequency for the terms in Keyword column.
    
     Args:
        dataFrame (dataFrame, required): dataFrame with news in PySophia2 structure
        
    Returns:
        Dict: it has a Counter with the times that each word appears on the keywords for the news.
    """
    
    if "keywords" not in df:
        get_keywords(df)
    concept_freq_total=Counter({})

    for index, row in df.iterrows():
        arreglo=row["keywords"].split(",")
        arreglo.pop()
        arreglo=list(dict.fromkeys(arreglo))#elimina los duplicados para contar una vez por noticia cada palabra
        concept_freq = Counter(arreglo)
        concept_freq_total = concept_freq_total + concept_freq
    return {"total_news":len(df),"total_elements": sum(concept_freq_total.values()), "dictionary": concept_freq_total}

def group_by_date(dataset, granularity="month"):
    """Group a set of news by a period.
    
     Args:
        dataset (dataFrame, required): dataFrame with news in PySophia2 structure
        granularity (string, required): could be day/month/year. Default: Month
        
    Returns:
        group_by: return a pandas groupby struct with all the news grouped by a period.
    """
    if granularity == "month":
        return dataset.groupby(by=["month","year"])
    if granularity == "year":
        return dataset.groupby(by=["year"])
    if granularity == "day":
        return dataset.groupby(by=["date"])
    
    
def calculateDist(dataset,group_by="month"):
    """Calculte the frequency distribution for a pysophia2 dataset grouped in periods.
    
     Args:
        dataset (dataFrame, required): dataFrame with news in PySophia2 structure
        group_by (string, required): could be day/month/year. Default: Month
        
    Returns:
        Sophia2WordDistributionStruct: a struct with the frecuency of keywords for each periods in a set of news.
    """
    if "keywords" not in dataset:
        get_keywords(dataset)
    dataset_=dataset.copy()
    dataset_["month"]=dataset_['date'].apply(lambda x: x.split("-")[1])
    dataset_["year"]=dataset_['date'].apply(lambda x: x.split("-")[0])
    gb_dataset=group_by_date(dataset_, group_by)
    pySophia2WordDistribution = {}
    if group_by == "day":
        for key, ds in gb_dataset:
            pySophia2WordDistribution[datetime.strptime(key, '%Y-%m-%d').strftime("%Y/%m/%d")] = get_frequency(ds)
    elif group_by == "month":
        for (month,year), ds in gb_dataset:
            pySophia2WordDistribution[(str(year)+"/"+str(month))] = get_frequency(ds)
    elif group_by == "year":
        for year, ds in gb_dataset:
            pySophia2WordDistribution[str(year)] = get_frequency(ds)
    else: return "Wrong granularity parameter, please use 'day', 'month' or 'year' as granularity parameter instead of "+group_by
    
    return pySophia2WordDistribution

def view(word_distribution, words, annotate=False, normalize=True): 
    fig, ax = plt.subplots()
    """Graph the frecuency term for a set of words, it could be normalized by the news in each period, or calculated in total times appearances.
    
     Args:
        word_distribution (Sophia2WordDistribution, required): struct calculated through calculateDist function.
        words ([string], required): words to be displayed on the graph.
        annotate (bool,optional): show the point over the graph. Default: False.
        normalize=(bool, optional): allow to normalize the quantity of appearances by the total news in that period. Default: True.
        
    Returns:
        None
    """
    ngram_dictionary={}
    for palabra in words:
        ngram_dictionary[palabra]=[]
        keys=[]
        if normalize==True:
            for key in word_distribution:
                keys.append(key)
                ngram_dictionary[palabra].append(word_distribution[key]["dictionary"][palabra]/word_distribution[key]["total_news"])
        else:    
            for key in word_distribution:
                keys.append(key)
                ngram_dictionary[palabra].append(word_distribution[key]["dictionary"][palabra])
        ax.plot(keys,ngram_dictionary[palabra],label=palabra)
        ax.legend(loc=5, bbox_to_anchor=(1.3, 0.5), fancybox=True, shadow=True )
        ax.set_title("PySophia2 NGramViewer")
        if(annotate):
            for xy in zip(keys, ngram_dictionary[palabra]):                               
                ax.annotate(('%.2f' % xy[1]), xy=xy, textcoords='data')
    ax.set_xlabel('Date', fontsize=12)
    ax.set_ylabel('Frequency', fontsize=12)
    
    
def freq(word_distribution,top_n=None, words=None, graph=True):
    fig, ax = plt.subplots()
    """return the frequency for a top_n words or a arbitrary array of words. It can be graphed.
    
     Args:
        word_distribution (Sophia2WordDistribution, required): struct calculated through calculateDist function.
        top_n (int,optional): quantity of the top words you want to get.
        words ([string], required): list words.
        graph (bool, optional): activate or deactivate the graph.
        
    Returns:
        None
    """
    concept_freq_total=Counter({})
    total_news=0
    for a in word_distribution:
        concept_freq_total= concept_freq_total + word_distribution[a]["dictionary"]
        total_news=total_news+word_distribution[a]["total_news"]
    if top_n==None and words ==None:
        print("debes pasar como argumento top_n o un arreglo words")
    elif top_n!=None and words !=None:
        print("debes pasar como argumento top_n o un arreglo words, no ambos")
    elif top_n!=None:
        top=concept_freq_total.most_common(top_n)
        if graph:
        
            x = [x[0] for x in top]
            y = [x[1]/total_news for x in top]
            ax.bar(x,y)
            ax.set_title("Term frecuency top %i words" %(top_n))
        return top
    elif words!=None:
        y = []
        for word in words:
            y.append(concept_freq_total[word]/total_news)
        ax.bar(words,y)
        ax.set_title("Term frecuency bar graph")
        ax.set_ylabel('Frequency', fontsize=12)
        ax.set_xticks(rotation=90)
        
        





import pandas as pd
import mariadb
import sys
from elasticsearch import Elasticsearch



columns = ['id', 'country', 'media_outlet', 'url', 'title', 'text', 'date', 'year', 'journalist', 'month']

sql_vars={
    "__username__":"",
    "__password__": "",
    "__cursor__":None,
    "__host__": "45.79.130.8",
    "__port__": 14096,
    "__database__":"Sun"

}

elastic_vars = {
    "__ip__" : "45.56.113.162",
    "__port__": 9200,
    "__user__": "elastic",
    "__password__": "uZCxWGE35lD3lhc",
    "__es__": Elasticsearch(
        ["45.56.113.162:9200"],
        http_auth=("elastic",
                   "uZCxWGE35lD3lhc"),
        timeout=60
        )
    
}

############ 0 LOGGING #############
def set_elastic_vars(__ip__=None, __port__=None, __user__= None, __password__= None):
    """Set the variables used for ElasticSearch connection

    Args:
        __ip__ (int, optional): [ElasticSearch host ip]. Defaults to "None".
        __port__ (int, optional): [ElasticSearch port]. Defaults to "None".
        __user__ (str, optional): [ElastichSearch username]. Defaults to "None".
        __password__ (str, optional): [ElasticSearch password]. Defaults to "None".
    """
    arguments=locals()
    has_changed=False
    global elastic_vars
    for a,b in arguments.items():
        if (b is not None):
            elastic_vars[a]=b
            has_changed=True
    if has_changed:
        host= elastic_vars["__ip__"]+":"+str(elastic_vars["__port__"])
        elastic_vars["__es__"] = Elasticsearch([host],
                                               http_auth=(elastic_vars["__user__"],
                                                          elastic_vars["__password__"]))
    return 1

def login(__username__="", __password__="", __host__=None, __port__=None, __database__=None): #45.79.130.8 #45.56.109.120
    """login into Sun database through a mariadb sql adapter.

    Args:
        __username__ (str, optional): [description]. Defaults to "sophia2api".
        __password__ (str, required): [description]. Defaults to "".
        __host__ (str, optional): [description]. Defaults to "45.79.130.8".
        __port__ (int, optional): [description]. Defaults to 14096.
        __database__ (str, optional): [description]. Defaults to "Sun".
    """
    arguments=locals()
    global sql_vars

    for a,b in arguments.items():
        if (b is not None):
            sql_vars[a]=b
    
    return connect_sun()

def connect_sun():
    """connect to sun
    Returns:
        
    """
    global sql_vars
    try:
        conn = mariadb.connect(
            user=sql_vars["__username__"],
            password=sql_vars["__password__"],
            host=sql_vars["__host__"],
            port=sql_vars["__port__"],
            database=sql_vars["__database__"]
        )
        sql_vars["__cursor__"] = conn.cursor()
        return "successful connection"
    except mariadb.Error as e:
        print(f"Error connecting to MariaDB Platform: {e}")
        return "connection could not be established"

def __connection_validator():
    if sql_vars["__cursor__"] is None: sys.exit("connect to the database first with: db.set_credentials(password)")

############ 1 INFO #############

def list_countries():
    """provides a list with all the countries that currently have media outlets in Sun.

    Returns:
        [string]: [country_name]
    """
    __connection_validator()
    sql_vars["__cursor__"].execute("SELECT DISTINCT country FROM news")
    return [x[0] for x in sql_vars["__cursor__"].fetchall()]

def list_media_outlets(countries=["all"]): 
    """provides a list with all media outlets that currently have media outlets in Sun.

    Returns:
        [string]: [media_outlet]
    """
    pd.set_option("display.max_rows",None)
    __connection_validator()
    if "all" in countries: countries=list_countries()
    media_outlets=[]
    for country in countries:
        sql_vars["__cursor__"].execute("SELECT DISTINCT media_outlet FROM news WHERE country=?",(country,))
        for media_outlet in sql_vars["__cursor__"].fetchall():
            media_outlets.append((country, media_outlet[0]))
    return pd.DataFrame(media_outlets,columns=["country", "media_outlet_name"])


def stats_countries():
    """Provides a list the countries and it's news quantity in Sun.

    Returns:
        df: country;news_quantity
    """
    __connection_validator()
    sql_vars["__cursor__"].execute("SELECT country, count(id) FROM news GROUP BY country")
    return pd.DataFrame(sql_vars["__cursor__"].fetchall(),columns=["country", "quantity"])
    

def stats_countries_by_date(_from, _to):
    """Provides a list the countries and it's news quantity between a date.
    
     Args:
        __from (string, required): [ElasticSearch host ip]. It must be passed as a string with the following format: YYYY/MM/DD.
        __to (string, required): [ElasticSearch host ip]. It must be passed as a string with the following format: YYYY/MM/DD.
        
    Returns:
        df: country;news_quantity
    """
    __connection_validator()
    sql_vars["__cursor__"].execute("SELECT country, count(id) FROM news WHERE date >= ? AND date <= ? GROUP BY country",(_from,_to))
    return pd.DataFrame(sql_vars["__cursor__"].fetchall(),columns=["country", "quantity"])
    

def stats_media_outlet(country=None):
    """Provides a list the media outlets and it's news quantity in Sun.
        country (string, optional): country name.

    Returns:
        df: media_outlet;news_quantity
    """
    pd.set_option("display.max_rows",None)
    __connection_validator()
    if country is not None:
        sql_vars["__cursor__"].execute("SELECT country, media_outlet, count(id) FROM news WHERE country = ? GROUP BY media_outlet",(country,))
    else:
        sql_vars["__cursor__"].execute("SELECT country, media_outlet, count(id) FROM news GROUP BY media_outlet")
    return pd.DataFrame(sql_vars["__cursor__"].fetchall(),columns=["country", "media_outlet_name", "quantity"])


############ 2 DATASETS #############

def last_n(n=1):
    __connection_validator()
    sql_vars["__cursor__"].execute("SELECT *, month(date) as month FROM news ORDER BY ID DESC LIMIT ?" , (n,))
    return pd.DataFrame(sql_vars["__cursor__"].fetchall(), columns=columns)

def get_dataset(country, from_, to_, keyword=None):
    global elastic_vars
    if keyword==None or keyword=="":
                query = {
            "query": { 
                "bool": {
                    "filter":[
                        {"range":{
                            "date":{
                                "gte": from_,
                                "lt": to_
                                }
                            }
                        },
                        {"term":{
                            "country": country
                            }
                        }
                        ]
                    }
                }
            }
    else:
        query = {
            "query": { 
                "bool": {
                    "must": [{
                        "match": {
                            "text":keyword
                            }
                        }],
                    "filter":[
                        {"range":{
                            "date":{
                                "gte": from_,
                                "lt": to_
                                }
                            }
                        },
                        {"term":{
                            "country": country
                            }
                        }
                        ]
                    }
                }
            }        

    res = elastic_vars["__es__"].search(index="news", body=query, size=10000)
    n_news = res['hits']['total']['value']
    if n_news == 10000:
        print("Se encontraron mas de %d noticias, por favor acotar la fecha de busqueda" % n_news)
    else:
        print("Son %d noticias encontradas..." % n_news)
    
    data = {'id_news':[],'country':[],'media_outlet':[],'url':[],'title':[],'text':[],'date':[],'search':[]}
    df = pd.DataFrame(data)
    
    for hit in res['hits']['hits']:
        id_news = hit['_source']['id_news']
        country = hit['_source']['country']
        media_outlet = hit['_source']['media_outlet']
        url = hit['_source']['url']
        title = hit['_source']['title']
        text = hit['_source']['text']
        date = hit['_source']['date']
        search = keyword
    
        new_row = {'id_news':id_news, 'country':country, 'media_outlet':media_outlet, 'url':url, 'title':title, 'text':text, 'date':date, 'search':search}
    
        df = df.append(new_row, ignore_index=True)
    
    return df
    
def export_to_csv(df, name="default"):
    df.to_csv(name,sep=";")
    print("dataset saved as"+name+".csv")
    
## FUNCIONES CASO DE USO 3




import pathlib
from setuptools import setup

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()

# This call to setup() does all the work
setup(
    name="pytetete",
    version="0.0.38",
    description="Python library for news discourse analysis",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/TeamSophia2/pySophia2",
    author="Team Sophia2",
    author_email="mvernier@inf.uach.cl",
    license="MIT",
    classifiers=[
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8"
    ],
    packages=["pysophia2"],
    install_requires=["pandas", "mariadb", "elasticsearch", "spacy", "matplotlib", "numpy"],

)